@@ -1,14 +1,16 @@
 """HTTP Client library"""
 import json
-
+from .exceptions import handle_error
 
 try:
     # Python 3
     import urllib.request as urllib
     from urllib.parse import urlencode
+    from urllib.error import HTTPError
 except ImportError:
     # Python 2
     import urllib2 as urllib
+    from urllib2 import HTTPError
     from urllib import urlencode
 
 
@@ -45,6 +47,13 @@ class Response(object):
         """
         return self._headers
 
+    @property
+    def to_dict(self):
+        """
+        :return: dict of response from the API
+        """
+        return json.loads(self.body.decode('utf-8'))
+
 
 class Client(object):
     """Quickly and easily access any REST or REST-like API."""
@@ -52,7 +61,8 @@ class Client(object):
                  host,
                  request_headers=None,
                  version=None,
-                 url_path=None):
+                 url_path=None,
+                 append_slash=False):
         """
         :param host: Base URL for the api. (e.g. https://api.sendgrid.com)
         :type host:  string
@@ -74,6 +84,8 @@ class Client(object):
         self._url_path = url_path or []
         # These are the supported HTTP verbs
         self.methods = ['delete', 'get', 'patch', 'post', 'put']
+        # APPEND SLASH set
+        self.append_slash = append_slash
 
     def _build_versioned_url(self, url):
         """Subclass this function for your own needs.
@@ -97,6 +109,11 @@ class Client(object):
         while count < len(self._url_path):
             url += '/{0}'.format(self._url_path[count])
             count += 1
+
+        # add slash
+        if self.append_slash:
+            url += '/'
+
         if query_params:
             url_values = urlencode(sorted(query_params.items()), True)
             url = '{0}?{1}'.format(url, url_values)
@@ -119,11 +136,12 @@ class Client(object):
         :type name: string
         :return: A Client object
         """
-        url_path = self._url_path+[name] if name else self._url_path
+        url_path = self._url_path + [name] if name else self._url_path
         return Client(host=self.host,
                       version=self._version,
                       request_headers=self.request_headers,
-                      url_path=url_path)
+                      url_path=url_path,
+                      append_slash=self.append_slash)
 
     def _make_request(self, opener, request):
         """Make the API call and return the response. This is separated into
@@ -135,7 +153,12 @@ class Client(object):
         :type request: urllib.Request object
         :return: urllib response
         """
-        return opener.open(request)
+        try:
+            return opener.open(request)
+        except HTTPError as err:
+            exc = handle_error(err)
+            exc.__cause__ = None
+            raise exc
 
     def _(self, name):
         """Add variable values to the url.
@@ -173,7 +196,7 @@ class Client(object):
         if name in self.methods:
             method = name.upper()
 
-            def http_request(*args, **kwargs):
+            def http_request(*_, **kwargs):
                 """Make the API call
                 :param args: unused
                 :param kwargs:
@@ -181,10 +204,18 @@ class Client(object):
                 """
                 if 'request_headers' in kwargs:
                     self._update_headers(kwargs['request_headers'])
-                data = json.dumps(kwargs['request_body']).encode('utf-8')\
-                    if 'request_body' in kwargs else None
-                params = kwargs['query_params']\
-                    if 'query_params' in kwargs else None
+                if 'request_body' not in kwargs:
+                    data = None
+                else:
+                    # Don't serialize to a JSON formatted str if we don't have a JSON Content-Type
+                    if 'Content-Type' in self.request_headers:
+                        if self.request_headers['Content-Type'] != 'application/json':
+                            data = kwargs['request_body'].encode('utf-8')
+                        else:
+                            data = json.dumps(kwargs['request_body']).encode('utf-8')
+                    else:
+                        data = json.dumps(kwargs['request_body']).encode('utf-8')
+                params = kwargs['query_params'] if 'query_params' in kwargs else None
                 opener = urllib.build_opener()
                 request = urllib.Request(self._build_url(params), data=data)
                 if self.request_headers:
